#ifndef _SASBASE_H
#define	_SASBASE_H

#include "commdef.h"

#pragma pack(1)

enum _SAS_ATTRIBUTE_ITEMS
{
	SAS_READ_RECOVERED_WO_DELAY,	 
	SAS_READ_RECOVERED_W_DELAY,
	SAS_READ_RECOVERED_W_RETRY,
	SAS_READ_RECOVERED,
	SAS_READ_TOTAL_BYTES,
	SAS_READ_UNRECOVERED,
	SAS_WRITE_RECOVERED_WO_DELAY,
	SAS_WRITE_RECOVERED_W_DELAY,
	SAS_WRITE_RECOVERED_W_RETRY,
	SAS_WRITE_RECOVERED,
	SAS_WRITE_TOTAL_BYTES,
	SAS_WRITE_UNRECOVERED,
	SAS_VERIFY_RECOVERED_WO_DELAY,
	SAS_VERIFY_RECOVERED_W_DELAY,
	SAS_VERIFY_RECOVERED_W_RETRY,
	SAS_VERIFY_RECOVERED,
	SAS_VERIFY_UNRECOVERED,
	SAS_NON_MEDIUM_ERRORS,
	SAS_PLIST_COUNT,
	SAS_GLIST_COUNT,
	TOTAL_SAS_ATTR_ITEMS
};

enum _SMART_ITEMS
{
	SMART_READ_ERROR_RATE_1,	// read error rate
	SMART_SPINUP_TIME_3,		// spinup time
	SMART_REALLOCATION_5,		// reallocation
	SMART_SEEK_ERROR_RATE_7,	// seek error rate
	SMART_SPINUP_RETRY_10,		// spinup retry
	SMART_CALIBRATION_RETRY_11,	// calibration retry
	SMART_TEMPERATURE_194,		// temperature
	SMART_SSD_RESERVED_SPACE_232,
	SMART_SSD_WEARING_INDICATOR_233,
	TOTAL_SMART_ITEMS,
};

enum _SAS_HW
{
	SAS_HW_DISK,				// disks
	SAS_HW_VOLTAGE,				// voltages
	SAS_HW_FAN,					// fans
	SAS_HW_POWER,				// powers
	SAS_HW_TEMPERATURE,			// temperatures
	TOTAL_SAS_HW
};

typedef struct sSMART_DATA
{
	WORD    smIndex;
	BYTE    smStatus;
	BYTE    smThreshold;
} sSMART_DATA, *pSMART_DATA;

// SCSI attributes
typedef struct sSCSI_ATTR
{
	BYTE scsiChannel;				// channel for SCSI target (0/1), if == 0x80 see sSCSI_ATTR_EXT
	BYTE scsiId;					// SCSI ID for the volume
	BYTE scsiLun;					// LUN (0 if target is IDE)
	BYTE scsiTaggedQueuing;			// 1 --> queue enabled
	BYTE scsiCacheMode;				// 1 --> Cache enabled
	BYTE scsiSpeed;					// IDE(0:ATA33/1:ATA66/2:ATA100/3:ATA133)
} sSCSI_ATTR, *pSCSI_ATTR;

// SCSI attributes(for external RAID)
typedef struct sSCSI_ATTR_EXT
{
	DWORD   scsiChanMask;
	BYTE    scsiId;
	BYTE    scsiLun;
	BYTE    scsiTaggedQueuing;
	BYTE    scsiCacheMode;
	BYTE    scsiSpeed;
	BYTE	scsiRes[3];		// pad to DWORD boundary
} sSCSI_ATTR_EXT, *pSCSI_ATTR_EXT;

typedef struct sGUI_ENC_PROPERTY
{
	BYTE	gepVendor[8];			// vendor name
	BYTE	gepProduct[16];			// product name
	BYTE	gepRev[4];				// reversion 
	BYTE	gepEncType;				// enclosure type
	BYTE	gepEncnumber;			// enclosure#
	BYTE	gepEncDisks;			// number of disks
	BYTE	gepEncVoltages;			// number of voltages
	BYTE	gepEncFans;				// number of fans
	BYTE	gepEncPowers;			// number of powers
	BYTE	gepEncTemperatures;		// number of temperatures
	BYTE	gepRes[0x1D];			// reserved
} sGUI_ENC_PROPERTY, *pGUI_ENC_PROPERTY;


#define	MAX_ENCLOSURES			8
#define	MAX_PHY_PER_ENCLOSURE	32
#define	MAX_DEV_PER_ENCLOSURE	32

enum
{
	ENC_TYPE_NOT_INSTALLED,			// not installed
	ENC_TYPE_INTERNAL,				// built-in enclosure
	ENC_TYPE_SES2,					// ses2
	ENC_TYPE_SMP,					// smp
	ENC_TYPE_CTRL					// controller
};

#define	MAX_ELEMENT_STR_LEN		24
#define MAX_SENSORS				128

// constant of SES element definitions
// common status
// refers to SCSI Enclosure Services - 3 (SES-3) specification
#define SES_COMMON_STATUS_UNSUPPORTED	0x00		// Unsupported Status detection is not implemented for this element.
#define SES_COMMON_STATUS_OK			0x01		// Element is installed and no error conditions are known.
#define SES_COMMON_STATUS_CRITICAL		0x02		// Critical condition is detected.
#define SES_COMMON_STATUS_NONCRITICAL	0x03		// Noncritical condition is detected.
#define SES_COMMON_STATUS_UNRECOVERABLE	0x04        // Unrecoverable condition is detected.
#define SES_COMMON_STATUS_NOT_INSTALLED 0x05		// Element is not installed in enclosure.
#define SES_COMMON_STATUS_UNKNOWN       0x06        // Sensor has failed or element status is not available.
#define SES_COMMON_STATUS_NOT_AVAILABLE 0x07		// Element installed, no known errors, but the element has not been turned on or set into operation.
#define SES_COMMON_STATUS_NO_ACCESS     0x08        // The initiator port from which the RECEIVE DIAGNOSTIC RESULT command was received does not have access to this element.

// power
#define SES_POWER_FAIL				0x40000000

// fan(cooling)
#define SES_FAN_FAIL				0x40000000

// temperature
#define SES_OVER_TEMPERATURE		0x08000000

// voltage
#define SES_OVER_VOLTAGE			0x00000200
#define SES_UNDER_VOLTAGE			0x00000100

typedef struct sELEMENT
{
	BYTE	snNumberOfElements;								// number of elements
	BYTE	snElementStr[MAX_SENSORS][MAX_ELEMENT_STR_LEN];	// description of elements
	DWORD	snValue[MAX_SENSORS];							// the value of sensor
	DWORD	snRawValue[MAX_SENSORS];						// the SES RAW value of sensor, B3B2B1B0	
}sELEMENT, *pELEMENT;

typedef struct sSENSORS
{
	sELEMENT        disk;
	sELEMENT		voltage;
	sELEMENT		fan;
	sELEMENT		power;
	sELEMENT		temperature;	
	sELEMENT		battery;
} sSENSORS, *pSENSORS;

#pragma pack()

#endif



